package com.hyperspacehub.pushsample_example;
import android.annotation.TargetApi;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Build;
import android.util.Log;

import com.google.android.gms.location.LocationRequest;

import co.acoustic.mobile.push.sdk.api.MceApplication;
import co.acoustic.mobile.push.sdk.api.MceSdk;
import co.acoustic.mobile.push.sdk.api.MceSdkConfiguration;
import co.acoustic.mobile.push.sdk.api.notification.NotificationsPreference;
import co.acoustic.mobile.push.sdk.db.DefaultSdkDatabaseSecretKeyGenerator;
import co.acoustic.mobile.push.sdk.db.android.AndroidDatabaseImpl;
import co.acoustic.mobile.push.sdk.encryption.DefaultSdkEncryptionProvider;
import co.acoustic.mobile.push.sdk.util.Logger;

public class MainApplication extends  MceApplication{

    private static final boolean READ_CONFIG_FROM_ASSETS = true;
    public static final String MCE_SAMPLE_NOTIFICATION_CHANNEL_ID = "mce_sample_channel";

    @Override
    public void onCreate() {
        super.onCreate();
        Log.d("VersionTest", "v = ");
        MceSdk.getNotificationsClient().getNotificationsPreference().setSoundEnabled(getApplicationContext(), true);
        MceSdk.getNotificationsClient().getNotificationsPreference().setVibrateEnabled(getApplicationContext(), true);
        long[] vibrate = { 0, 100, 200, 300 };
        MceSdk.getNotificationsClient().getNotificationsPreference().setVibrationPattern(getApplicationContext(), vibrate);
        MceSdk.getNotificationsClient().getNotificationsPreference().setLightsEnabled(getApplicationContext(), true);
        int ledARGB = 0x00a2ff;
        int ledOnMS = 300;
        int ledOffMS = 1000;
        MceSdk.getNotificationsClient().getNotificationsPreference().setLights(getApplicationContext(), new int[]{ledARGB, ledOnMS, ledOffMS});

        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            createNotificationChannel(getApplicationContext());
        }
    }

    @Override
    protected MceSdkConfiguration getMceSdkConfiguration() {
        if(READ_CONFIG_FROM_ASSETS) {
            return null;
        } else {
            MceSdkConfiguration mceSdkConfiguration = new MceSdkConfiguration("gcqbTJWhch","564506389599");
            mceSdkConfiguration.setBaseUrl("https://sdk7.ibm.xtify.com");
            mceSdkConfiguration.setAutoInitialize(true);
            mceSdkConfiguration.setAutoReinitialize(true);

            mceSdkConfiguration.setGroupNotificationsByAttribution(true);
            mceSdkConfiguration.setInvalidateExistingUser(false);
            mceSdkConfiguration.setMessagingService(MceSdkConfiguration.MessagingService.fcm);

            mceSdkConfiguration.setSessionsEnabled(true);
            mceSdkConfiguration.setSessionTimeout(20);

            mceSdkConfiguration.setMetricTimeInterval(20);
            mceSdkConfiguration.setLogFile(false);
            mceSdkConfiguration.setLogLevel(Logger.LogLevel.error);
            mceSdkConfiguration.setLogIterations(1);
            mceSdkConfiguration.setLogIterationDurationInHours(0);
            mceSdkConfiguration.setLogBufferSize(10);


            mceSdkConfiguration.setUseFileImageCache(true);
            mceSdkConfiguration.setUseInMemoryImageCache(true);
            mceSdkConfiguration.setFileImageCacheCapacityInMB(100);
            mceSdkConfiguration.setInMemoryImageCacheCapacityInMB(10);

            MceSdkConfiguration.LocationConfiguration.SyncConfiguration syncConfiguration = mceSdkConfiguration.getLocationConfiguration().getSyncConfiguration();
            syncConfiguration.setLocationResponsiveness(300);
            syncConfiguration.setLocationSearchRadius(100000);
            syncConfiguration.setSyncInterval(300);
            syncConfiguration.setMaxLocationsForSearch(1);
            syncConfiguration.setMaxLocationsForSearch(20);

            MceSdkConfiguration.LocationConfiguration.RequestConfiguration requestConfiguration = mceSdkConfiguration.getLocationConfiguration().getRequestConfiguration();
            requestConfiguration.setFastestInterval(1000);
            requestConfiguration.setInterval(5000);
            requestConfiguration.setSmallestDisplacement(100);
            requestConfiguration.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);

            MceSdkConfiguration.LocationConfiguration.IBeaconConfiguration iBeaconConfiguration = mceSdkConfiguration.getLocationConfiguration().getiBeaconConfiguration();
            iBeaconConfiguration.setUuid("YOUR UUID");
            iBeaconConfiguration.setBeaconForegroundScanDuration(5);
            iBeaconConfiguration.setBeaconForegroundScanInterval(30);
            iBeaconConfiguration.setBeaconBackgroundScanDuration(30);
            iBeaconConfiguration.setBeaconBackgroundScanInterval(300);

            MceSdkConfiguration.DatabaseConfiguration databaseConfiguration = mceSdkConfiguration.getDatabaseConfiguration();
            databaseConfiguration.setDatabaseImplClassName(AndroidDatabaseImpl.class.getName());
            databaseConfiguration.setEncrypted(false);
            databaseConfiguration.setEncryptionProviderClassName(DefaultSdkEncryptionProvider.class.getName());
            databaseConfiguration.setKeyGeneratorClassName(DefaultSdkDatabaseSecretKeyGenerator.class.getName());
            databaseConfiguration.setKeyRotationIntervalInDays(30);

            return mceSdkConfiguration;
        }

    }

    private static final String PREFS_NAME = "IBM_MCE_SAMPLE";

    private static SharedPreferences getSharedPref(Context context) {
        return context.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
    }

    private static SharedPreferences.Editor getEditor(Context context) {
        return getSharedPref(context).edit();
    }

    @TargetApi(26)
    private static void createNotificationChannel(Context context) {
        NotificationManager notificationManager =
                (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
        NotificationChannel channel = notificationManager.getNotificationChannel(MCE_SAMPLE_NOTIFICATION_CHANNEL_ID);
        if(channel == null) {
            int importance = NotificationManager.IMPORTANCE_HIGH;
            channel = new NotificationChannel(MCE_SAMPLE_NOTIFICATION_CHANNEL_ID, "SDK Notification Channel", importance);
            channel.setDescription("This is the notification channel for the SDK sample application");
            channel.setShowBadge(true);
            NotificationsPreference notificationsPreference = MceSdk.getNotificationsClient().getNotificationsPreference();
            notificationsPreference.setNotificationChannelId(context, MCE_SAMPLE_NOTIFICATION_CHANNEL_ID);
            notificationManager.createNotificationChannel(channel);
        }
    }
}
